//-----------------------------------------------------------------------------
//
// RAlert.cpp
//
// Reusable alert class.
//
// Adds following features to BAlert:
//
// Centers alert over window or on main screen.
// Changes to hand cursor.
// Provides simple info and error message functions.
//
//-----------------------------------------------------------------------------

#include "RAlert.h"
#include "RUtil.h"

B_DEFINE_CLASS_INFO(RAlert, BAlert);

///RAlert
//-----------------------------------------------------------------------------

RAlert::RAlert(const char *text,
	const char *button1,
	const char *button2,
	const char *button3,
	alert_type type)
	: BAlert("", text, button1, button2, button3, B_WIDTH_AS_USUAL, type)
{
	BRect a = Frame();
	screen_info screenInfo;
	get_screen_info(&screenInfo);
	CenterRectOverScreenRect(a, screenInfo.frame);
	MoveTo(a.left, a.top);
}

RAlert::RAlert(const char *text,
	const BRect centerOver,
	const char *button1,
	const char *button2,
	const char *button3,
	alert_type type)
	: BAlert("", text, button1, button2, button3, B_WIDTH_AS_USUAL, type)
{
	BRect a = Frame();
	CenterRectOverScreenRect(a, centerOver);
	MoveTo(a.left, a.top);
}

///Go
//-----------------------------------------------------------------------------

long RAlert::Go()
{
	be_app->SetCursor(B_HAND_CURSOR);
	return inherited::Go();
}

///ErrMsg
//-----------------------------------------------------------------------------

void RAlert::ErrMsg(char *str)
{
	RAlert *alert = new RAlert(str, "OK", NULL, NULL, B_STOP_ALERT);
	alert->Go();
}

void RAlert::ErrMsg(char *str, BRect centerOver)
{
	RAlert *alert = new RAlert(str, centerOver, "OK", NULL, NULL, B_STOP_ALERT);
	alert->Go();
}

///InfoMsg
//-----------------------------------------------------------------------------

void RAlert::InfoMsg(char *str)
{
	RAlert *alert = new RAlert(str);
	alert->Go();
}

void RAlert::InfoMsg(char *str, BRect centerOver)
{
	RAlert *alert = new RAlert(str, centerOver);
	alert->Go();
}
