//-----------------------------------------------------------------------------
//
// RFile.cpp
//
// Reusable file class.
//
// Adds following features to BFile:
//
// A Ref() function.
// ReadToMem() and WriteFromMem() functions.
//
//-----------------------------------------------------------------------------

#include "RFile.h"

B_DEFINE_CLASS_INFO(RFile, BFile);

///RFile
//-----------------------------------------------------------------------------

RFile::RFile()
{
}

RFile::RFile(record_ref ref)
	:BFile(ref)
{
}

///Ref()
//-----------------------------------------------------------------------------

RFile::~RFile()
{
}

///Ref
//-----------------------------------------------------------------------------

record_ref RFile::Ref()
{
	return Record()->Ref();
}

///ReadToMem
//-----------------------------------------------------------------------------

void *RFile::ReadToMem(long *size)
{
	// Returns a pointer to the read data.
	// Data is 0 terminated, and size is returned if it's not NULL.
	// Returned size does not include 0 terminator.
	
	char *data = NULL;
	OpenData();
	if (!Error()) {
		long len = DataSize();
		if (!Error()) {
			data = new char[len+1];
			Read(data, len);
			if (!Error()) {
				data[len] = 0;
				if (size) *size = len;
			}
		}
		CloseData();
	}
	if (Error()) {
		delete [] data;
		return NULL;
	} else {
		return data;
	}
}

///WriteFromMem
//-----------------------------------------------------------------------------

void RFile::WriteFromMem(const void *data, const long size)
{
	OpenData();
	if (!Error()) {
		SetDataSize(0);
		if (!Error()) Write(data, size);
		CloseData();
	}
}

void RFile::WriteFromMem(const void *data)
{
	// Data must must be 0-terminated.
	
	WriteFromMem(data, strlen((char*)data));
}
