//-----------------------------------------------------------------------------
//
// RWriteLocker.cpp
//
// This class implements a lock which permits multiple simultaneous reader
// threads but guarantees exclusive access to writer threads.
//
// The algorithm is adapted from "Operating Systems Concepts", second
// edition, by James L. Perterson and Abraham Silberschatz.
//
//-----------------------------------------------------------------------------

#include "RWriteLocker.h"

B_DEFINE_CLASS_INFO(RWriteLocker, BObject);

///RWriteLocker
//-----------------------------------------------------------------------------

RWriteLocker::RWriteLocker()
	: numReaders(0)
{
	entry = create_sem(1, "RWriteLocker entry semaphore");
	write = create_sem(1, "RWriteLocker write semaphore");
}

///~RWriteLocker
//-----------------------------------------------------------------------------

RWriteLocker::~RWriteLocker()
{
	delete_sem(entry);
	delete_sem(write);
}

///ReadLock
//-----------------------------------------------------------------------------

void RWriteLocker::ReadLock()
{
	acquire_sem(entry);
	if (atomic_add(&numReaders, 1) == 0) acquire_sem(write);
	release_sem(entry);
}

///ReadUnlock
//-----------------------------------------------------------------------------

void RWriteLocker::ReadUnlock()
{
	if (atomic_add(&numReaders, -1) == 1) release_sem(write);
}

///WriteLock
//-----------------------------------------------------------------------------

void RWriteLocker::WriteLock()
{
	acquire_sem(entry);
	acquire_sem(write);
	release_sem(entry);
}

///WriteUnlock
//-----------------------------------------------------------------------------

void RWriteLocker::WriteUnlock()
{
	release_sem(write);
}
