//-----------------------------------------------------------------------------
//
// TApplication.cpp
//
// BetterEdit application class.
//
//-----------------------------------------------------------------------------

#include "TApplication.h"
#include "TGlobals.h"
#include "RWindowsMenu.h"
#include "TSearch.h"
#include "TDepend.h"
#include "TCompile.h"
#include "RAlert.h"
#include "TWindow.h"
#include "TUtil.h"
#include "RFile.h"
#include "RDirectory.h"
#include "TProjectInfo.h"
#include "RPanelInfo.h"

B_DEFINE_CLASS_INFO(TApplication, BApplication);

///main
//-----------------------------------------------------------------------------

main (short argc, char **argv)
{
	my_app = new TApplication();
	DEBUG_ON();
	my_app->Run();
	windows_menu_blooper->Quit();
	search_blooper->Quit();
	thread_id id = depend_blooper->Thread();
	depend_blooper->WriteIncludesAndQuit();
	long result;
	wait_for_thread(id, &result);
	compile_blooper->Quit();
	delete(my_app);
	return 0;
}

///TApplication
//-----------------------------------------------------------------------------

TApplication::TApplication()
	: BApplication(SIGNATURE)
{
	// Create the global blooper objects.
	
	windows_menu_blooper = new RWindowsMenu("menuBar", "Windows", 
		WINDOWS_MENU, "wind");
	search_blooper = new TSearch();
	depend_blooper = new TDepend();
	compile_blooper = new TCompile();
	
	// Initialize the file/save panel frame.
	
	screen_info screenInfo;
	get_screen_info(&screenInfo);
	BRect s = screenInfo.frame;
	panel_info.Lock();
	panel_info.frame.left = s.left + (s.right - s.left - 400)/2;
	panel_info.frame.right = panel_info.frame.left + 400;
	panel_info.frame.top = s.top + 100;
	panel_info.frame.bottom = s.bottom-90;
	panel_info.Unlock();

	// Create the main menu.

	BPopUpMenu *menu = new BPopUpMenu("", FALSE, FALSE);
	BMenuItem *item;	
	item = new BMenuItem("About BetterEdit...", 
		new BMessage(B_ABOUT_REQUESTED));
	menu->AddItem(item);
		
	menu->AddSeparatorItem();	
	item = new BMenuItem("New", new BMessage(NEW), 'N');
	menu->AddItem(item);	
	item = new BMenuItem("Open...", new BMessage(OPEN), 'O');
	menu->AddItem(item);	
	item = new BMenuItem("Save All", new BMessage(SAVE_ALL), 
		'S', B_SHIFT_KEY);
	menu->AddItem(item);
	
	menu->AddSeparatorItem();
	item = new BMenuItem("Open Project File...", 
		new BMessage(OPEN_PROJECT_FILE), 'P');
	item->SetEnabled(FALSE);
	menu->AddItem(item);	
	item = new BMenuItem("Open Be Header File...", 
		new BMessage(OPEN_BE_HEADER), 'B');
	menu->AddItem(item);
	
	menu->AddSeparatorItem();
	item = new BMenuItem("Make", new BMessage(MAKE), 'M');
	menu->AddItem(item);
	item = new BMenuItem("Run", new BMessage(RUN), 'R');
	menu->AddItem(item);
	
	menu->AddSeparatorItem();
	item = new BMenuItem("Make Clean", new BMessage(MAKE_CLEAN), 'M',
		B_SHIFT_KEY);
	menu->AddItem(item);
	item = new BMenuItem("Create Makefile", new BMessage(CREATE_MAKE_FILE),
		'K', B_SHIFT_KEY);
	menu->AddItem(item);
	item = new BMenuItem("Set Project...", new BMessage(SET_PROJECT),
		'T', B_SHIFT_KEY);
	menu->AddItem(item);
	
	menu->AddSeparatorItem();
	item = new BMenuItem("Convert Files...", new BMessage(CONVERT_FILES), 'F',
		B_SHIFT_KEY);
	menu->AddItem(item);
	
	menu->AddSeparatorItem();
	item = new BMenuItem("Quit BetterEdit", 
		new BMessage(B_QUIT_REQUESTED), 'Q');
	menu->AddItem(item);
	menu->SetTarget(be_app);
	SetMainMenu(menu);
}

///MenusWillShow
//-----------------------------------------------------------------------------

void TApplication::MenusWillShow()
{
	BPopUpMenu *menu = MainMenu();
	BMenuItem *item;
	item = menu->FindItem(MAKE);
	item->SetEnabled(project_info.haveProject);
	item = menu->FindItem(RUN);
	item->SetEnabled(project_info.haveProject);
	item = menu->FindItem(MAKE_CLEAN);
	item->SetEnabled(project_info.haveProject);
	item = menu->FindItem(CREATE_MAKE_FILE);
	item->SetEnabled(project_info.haveProject);
}

///AboutRequested
//-----------------------------------------------------------------------------

void TApplication::AboutRequested()
{
	char str[] = 
		"BetterEdit\n\n"
		"Version 1.0 (for DR6)\n\n"
		"A simple BeBox text editor and IDE.\n\n"
		"John Norstad.";
	RAlert *alert = new RAlert(str, "OK");
	alert->Go();
}

///MessageReceived
//-----------------------------------------------------------------------------

void TApplication::MessageReceived(BMessage *msg)
{
	switch (msg->what) {
		case NEW:
			{			
				new TWindow();
			}
			break;
		case OPEN:
			{			
				msg = new BMessage(B_REFS_RECEIVED);
				panel_info.Lock();
				if (panel_info.dir.database >= 0)
					msg->AddRef("directory", panel_info.dir);
				msg->AddRect("frame", panel_info.frame);
				panel_info.Unlock();
				RunFilePanel(NULL, NULL, FALSE, msg);
			}
			break;
		case SAVE_ALL:
			{			
				SaveAll();
			}
			break;
		case OPEN_PROJECT_FILE:
			{		
				record_ref ref = msg->FindRef("ref");	
				msg = new BMessage(B_REFS_RECEIVED);
				msg->AddRef("directory", ref);
				panel_info.Lock();
				msg->AddRect("frame", panel_info.frame);
				panel_info.Unlock();
				msg->AddBool("Don't save panel dir", TRUE);
				RunFilePanel(NULL, NULL, FALSE, msg);
			}
			break;
		case OPEN_BE_HEADER:
			{			
				record_ref ref;
				project_info.ReadLock();
				long err = get_ref_for_path(project_info.beHeaders, &ref);
				project_info.ReadUnlock();
				if (!err) {
					msg = new BMessage(B_REFS_RECEIVED);
					msg->AddRef("directory", ref);
					panel_info.Lock();
					msg->AddRect("frame", panel_info.frame);
					panel_info.Unlock();
					msg->AddBool("Open read-only", TRUE);
					msg->AddBool("Don't save panel dir", TRUE);
					RunFilePanel(NULL, NULL, FALSE, msg);
				}
			}
			break;
		case MAKE:
			{			
				HideAllPanels();
				if (!SaveAll(TRUE)) return;
				depend_blooper->Make();
			}
			break;
		case RUN:
			{			
				HideAllPanels();
				if (!SaveAll(TRUE)) return;
				depend_blooper->MakeAndRun();
			}
			break;
		case MAKE_CLEAN:
			{			
				MakeClean();
			}
			break;
		case CREATE_MAKE_FILE:
			{			
				if (!SaveAll(TRUE)) return;
				depend_blooper->CreateMakeFile();
			}
			break;
		case SET_PROJECT:
			{
				msg = new BMessage(B_REFS_RECEIVED);
				panel_info.Lock();
				if (panel_info.dir.database >= 0)
					msg->AddRef("directory", panel_info.dir);
				msg->AddRect("frame", panel_info.frame);
				panel_info.Unlock();
				// DR5 hack: Need bogus filter to force file panel
				// to display only directories, not files.
				msg->AddString("filter", "Directories only");
				msg->AddLong("Directories only", 1234);
				msg->AddBool("Set project", TRUE);
				RunFilePanel("Set BetterEdit Primary Project Directory", 
					"Set", TRUE, msg);
			}
			break;
		case CONVERT_FILES:
			{			
				msg = new BMessage(CONVERT_FILES_DOIT);
				panel_info.Lock();
				if (panel_info.dir.database >= 0)
					msg->AddRef("directory", panel_info.dir);
				msg->AddRect("frame", panel_info.frame);
				panel_info.Unlock();
				msg->AddString("filter", "Text files only");
				msg->AddString("filter", "All files");
				msg->AddLong("Text files only", 'TEXT');
				msg->AddLong("All files", 0);
				RunFilePanel("Convert Files to BetterEdit", 
					"Convert", FALSE, msg);
			}
			break;	
		case CONVERT_FILES_DOIT:
			{
				ConvertFiles(msg);
			}
			break;		
		case WINDOWS_MENU:
			{			
				TWindow *wind = (TWindow*)msg->FindObject("wind");
				if (wind->Lock()) {
					wind->Activate();
					wind->Unlock();
				}
			}
			break;
		case HIDE_ALL_PANELS:
			HideAllPanels();
			break;
		case ADD_PROJECT_OPEN_COMMANDS:
			AddProjectOpenCommands();
			break;
		default:
			inherited::MessageReceived(msg);
			break;
	}
}

///ReadyToRun
//-----------------------------------------------------------------------------

void TApplication::ReadyToRun()
{
	if (RWindow::CountWindows() == 0) new TWindow();
}

///RefsReceived
//-----------------------------------------------------------------------------

void TApplication::RefsReceived(BMessage *msg)
{
	ulong type;
	long count;
	msg->GetInfo("refs", &type, &count);
	TWindow *wind;
	
	if (msg->FindBool("Set project")) {
		if (count == 1) {
			record_ref ref = msg->FindRef("refs", 0);
			if (ref.database >= 0 && does_ref_conform(ref, "Folder")) {
				depend_blooper->SetProject(ref);
			}
		}
		return;
	}
	
	for (long i = --count; i >= 0; i--) {
	
		record_ref ref = msg->FindRef("refs", i);
		
		if (ref.database >=0) {
		
			if (does_ref_conform(ref, "File")) {
			
				// Check to see if file is already open. If it is,
				// bring its window to the front.
				
				bool windFound = FALSE;
				BList windowList = RWindow::WindowList();
				for (long i = 0; 
					wind = (TWindow*)windowList.ItemAt(i); 
					i++)
				{
					if (wind->Lock()) {
						record_ref r = wind->Ref();
						if (r.database == ref.database && 
							r.record == ref.record) 
						{
							wind->Activate();
							windFound = TRUE;
						}
						wind->Unlock();
					}
					if (windFound) break;
				}
				
				// Read the file and open it in a new window if it 
				// wasn't already open.
				
				if (!windFound) {
					RFile file(ref);
					long size;
					void *data = file.ReadToMem(&size);
					char title[B_FILE_NAME_LENGTH+1];
					file.GetName(title);
					wind = new TWindow(data, size, 
						msg->FindBool("Open read-only"), 
						ref.database, ref.record, title);
					delete [] data;
				}
				
				// Save the file panel directory position.
				
				if (!msg->FindBool("Don't save panel dir")) {
					BFile file(ref);
					RDirectory parent;
					file.GetParent(&parent);
					panel_info.Lock();
					panel_info.dir = parent.Ref();
					panel_info.Unlock();
				}
				
				// If the message has a line number parameter, it's a compiler
				// error message sent by TCompile. Copy the error message
				// parameters to a go to line message and send it to 
				// the window's text view.
				
				if (msg->HasLong("line")) {
					if (wind->Lock()) {
						BMessage *newMsg = new BMessage(GO_TO_LINE_DOIT);
						newMsg->AddLong("line", 
							msg->FindLong("line"));
						newMsg->AddString("emess1", 
							msg->FindString("emess1"));
						newMsg->AddString("emess2", 
							msg->FindString("emess2"));
						newMsg->AddString("emess3", 
							msg->FindString("emess3"));
						wind->PostMessage(newMsg,
							(BReceiver*)wind->FindView("textView"));
						wind->Unlock();
					}
				}
			}
		}
		
	}
}

///FilePanelClosed
//-----------------------------------------------------------------------------

void TApplication::FilePanelClosed(BMessage *msg)
{
	panel_info.Lock();
	panel_info.frame = msg->FindRect("frame");
	panel_info.Unlock();
	// panel_info.dir not set here - set in RefsReceived instead.
}

///SaveAll
//-----------------------------------------------------------------------------

bool TApplication::SaveAll(bool onlySourceFiles)
{
	bool allSaved = TRUE;
	BList windowList = RWindow::WindowList();
	for (long i = 0; 
		TWindow *wind = (TWindow*)windowList.ItemAt(i); i++)
	{
		if (wind->Lock()) {
			record_ref ref = wind->Ref();
			if (ref.database >= 0) {
				if (!onlySourceFiles || IsDotCpp(ref) || IsDotH(ref) ||
					IsProjectFile(ref)) allSaved = wind->Save(SAVE);
			}
			wind->Unlock();
		}
		if (!allSaved) break;
	}
	return allSaved;
}

///MakeClean
//-----------------------------------------------------------------------------

void TApplication::MakeClean()
{
	project_info.ReadLock();
	if (!project_info.haveProject) return;
	RDirectory obj;
	long i = 0;
	while (project_info.GetObjDirectory(i++, &obj) == B_NO_ERROR) {
		RFile file;
		long j = 0;
		while (obj.GetFile(j, &file) == B_NO_ERROR) obj.Remove(&file);
	}
	project_info.ReadUnlock();
}

///HideAllPanels
//-----------------------------------------------------------------------------

void TApplication::HideAllPanels()
{
	BList windowList = RWindow::WindowList();
	for (long i = 0; 
		TWindow *wind = (TWindow*)windowList.ItemAt(i); i++)
	{
		if (wind->Lock()) {
			record_ref ref = wind->Ref();
			wind->HidePanel();
			wind->Unlock();
		}
	}
}

///ConvertFiles
//-----------------------------------------------------------------------------

void TApplication::ConvertFiles(BMessage *msg)
{
	ulong type;
	long count;
	msg->GetInfo("refs", &type, &count);
	
	for (long i = --count; i >= 0; i--) {
		record_ref ref = msg->FindRef("refs", i);
		if (ref.database >=0) {
			if (does_ref_conform(ref, "File")) {
				BFile file(ref);
				file.SetTypeAndApp('TEXT', SIGNATURE);
			}
		}
	}
}

///AddProjectOpenCommands
//-----------------------------------------------------------------------------

void TApplication::AddProjectOpenCommands()
{
	BPopUpMenu *mainMenu = MainMenu();
	long index;
	while (BMenuItem *item = mainMenu->FindItem(OPEN_PROJECT_FILE)) {
		index = mainMenu->IndexOf(item);
		mainMenu->RemoveItem(item);
		delete item;
	}
	project_info.ReadLock();
	long i = 0;
	RDirectory dir;
	while (project_info.GetDirectory(i++, &dir) == B_NO_ERROR) {
		char name[B_FILE_NAME_LENGTH+1];
		dir.GetName(name);
		char label[256];
		sprintf(label, "Open %s File...", name);
		BMessage *msg = new BMessage(OPEN_PROJECT_FILE);
		msg->AddRef("ref", dir.Ref());
		BMenuItem *item;
		if (i == 1) {
			item = new BMenuItem(label, msg, 'P');
		} else if (i == 2) {
			item = new BMenuItem(label, msg, 'P', B_SHIFT_KEY);
		} else {
			item = new BMenuItem(label, msg);
		}
		item->SetTarget(be_app);
		mainMenu->AddItem(item, index);
		index++;
	}
	project_info.ReadUnlock();
}
