//-----------------------------------------------------------------------------
//
// TFindDlg.cpp
//
// "Find" dialog class.
//
//-----------------------------------------------------------------------------

#include "TFindDlg.h"
#include "TFindInfo.h"
#include "RGlobals.h"
#include "RTextView.h"
#include "RUtil.h"
#include "TGlobals.h"
#include "TProjectInfo.h"

B_DEFINE_CLASS_INFO(TFindDlg, BWindow);

///TFindDlg
//-----------------------------------------------------------------------------

TFindDlg::TFindDlg(TWindow *parent)
	: BWindow(BRect(0, 0, 400, 290), "", B_MODAL_WINDOW, B_NOT_RESIZABLE),
	parent(parent)
{
	Lock();
	
	parentRef = parent->Ref();	
	
	BRect bounds = Bounds();
	
	// Center dialog over parent window.
	
	BRect a = Frame();
	CenterRectOverScreenRect(a, parent->Frame());
	MoveTo(a.left, a.top);
	
	find_info.Lock();
	
	caseSensitive = find_info.caseSensitive;
	wholeWord = find_info.wholeWord;
	wrap = find_info.wrap;
	forward = find_info.forward;
	scope = find_info.scope;
	
	haveProjectDir = project_info.haveProject;
	if (!haveProjectDir && scope != FIND_THIS_FILE &&
		scope != FIND_BE_HEADER_FILES && scope != FIND_STD_C_HEADER_FILES)
			scope = FIND_THIS_FILE;
	
	// Make the background light gray.
	
	BView *top = new BView(bounds, "", B_FOLLOW_NONE, B_WILL_DRAW);
	AddChild(top);
	top->SetViewColor(light_gray_background);
	
	// OK button.
		
	BRect okButtonRect(bounds.right-90, bounds.bottom-35,
		bounds.right-20, bounds.bottom-10);
	BButton *okButton = new BButton(okButtonRect, "",
		"OK", new BMessage('okok'));
	top->AddChild(okButton);	
	SetDefaultButton(okButton);
	
	// Cancel button.
	
	BRect cancelButtonRect(210, okButtonRect.top, 
		280, okButtonRect.bottom);
	BButton *cancelButton = new BButton(cancelButtonRect, "cancel",
		"Cancel", new BMessage('cncl'));
	top->AddChild(cancelButton);
	
	// Search string.
	
	BRect searchLabelRect(10, 20, 60, 36);
	MakeTextEntryField(searchLabelRect, "Search", top, "search", 
		TRUE, find_info.searchString);
	
	// Replace string.
	
	BRect replaceLabelRect = searchLabelRect;
	replaceLabelRect.OffsetBy(0, 30);
	MakeTextEntryField(replaceLabelRect, "Replace", top, "replace",
		FALSE, find_info.replaceString);
	
	// Options.
	
	BRect optFrameRect(20, replaceLabelRect.bottom+20,
		180, replaceLabelRect.bottom+101);
	BBox *opt = new BBox(optFrameRect, "", B_FOLLOW_NONE);
	top->AddChild(opt);
	opt->SetLowColor(light_gray_background);
	opt->SetViewColor(light_gray_background);
	opt->SetFontName("Emily");
	opt->SetLabel("Options");
	
	BRect boxFrame = opt->Bounds();
	boxFrame.InsetBy(25, 10);
	boxFrame.top = 20;
	boxFrame.bottom = boxFrame.top+11;
	
	MakeOptBox(&boxFrame, "Case sensitive", 'sens', opt,
		find_info.caseSensitive);
	MakeOptBox(&boxFrame, "Whole word", 'word', opt,
		find_info.wholeWord);
	wrapCheckBox = MakeOptBox(&boxFrame, "Wrap", 'wrap', opt,
		find_info.wrap);
		
	if (scope != FIND_THIS_FILE) wrapCheckBox->SetEnabled(FALSE);
	
	// Direction.
	
	BRect dirFrameRect(20, optFrameRect.bottom+10, 
		180, optFrameRect.bottom+74);
	BBox *dir = new BBox(dirFrameRect, "", B_FOLLOW_NONE);
	top->AddChild(dir);
	dir->SetLowColor(light_gray_background);
	dir->SetViewColor(light_gray_background);
	dir->SetFontName("Emily");
	dir->SetLabel("Direction");
	
	BRect dirRadioFrame = dir->Bounds();
	dirRadioFrame.InsetBy(25, 10);
	dirRadioFrame.top = 20;
	dirRadioFrame.bottom = dirRadioFrame.top+11;
	
	MakeDirRadio(&dirRadioFrame, "Forward", dir, TRUE);
	MakeDirRadio(&dirRadioFrame, "Backward", dir, FALSE);
	
	// Scopes.
	
	BRect scopeFrameRect(cancelButtonRect.left, replaceLabelRect.bottom+20,
		bounds.right-20, replaceLabelRect.bottom+169);
	BBox *scopeBox = new BBox(scopeFrameRect, "", B_FOLLOW_NONE);
	top->AddChild(scopeBox);
	scopeBox->SetLowColor(light_gray_background);
	scopeBox->SetViewColor(light_gray_background);
	scopeBox->SetFontName("Emily");
	scopeBox->SetLabel("Scope");
	
	BRect scopeRadioFrame = scopeBox->Bounds();
	scopeRadioFrame.InsetBy(25, 10);
	scopeRadioFrame.top = 20;
	scopeRadioFrame.bottom = scopeRadioFrame.top+11;
	
	MakeScopeRadio(&scopeRadioFrame, "This file only", 'cur ', scopeBox,
		FIND_THIS_FILE);
	MakeScopeRadio(&scopeRadioFrame, ".cpp files", 'cpp ', scopeBox,
		FIND_CPP_FILES);
	MakeScopeRadio(&scopeRadioFrame, ".h files", 'h   ', scopeBox,
		FIND_H_FILES);
	MakeScopeRadio(&scopeRadioFrame, ".cpp & .h files", 'cpph', scopeBox,
		FIND_CPP_AND_H_FILES);
	MakeScopeRadio(&scopeRadioFrame, "Text files", 'text', scopeBox,
		FIND_TEXT_FILES);
	MakeScopeRadio(&scopeRadioFrame, "Be header files", 'be  ', scopeBox,
		FIND_BE_HEADER_FILES);	
	MakeScopeRadio(&scopeRadioFrame, "Std C header files", 'stdc', scopeBox,
		FIND_STD_C_HEADER_FILES);	
		
	Show();		
	
	find_info.Unlock();
	Unlock();
}

///   MakeTextEntryField
//-----------------------------------------------------------------------------

void TFindDlg::MakeTextEntryField(BRect labelRect, const char *label,
	BView *container, const char *name, bool makeFocus, 
	const char *initValue)
{
	BRect bounds = Bounds();

	BStringView *stringView = new BStringView(labelRect, "", label);
	stringView->SetAlignment(B_ALIGN_RIGHT);
	container->AddChild(stringView);
	stringView->SetLowColor(light_gray_background);
	stringView->SetFontName("Emily");

	BRect frameRect(labelRect.right+6, labelRect.top, 
		bounds.right-20, labelRect.bottom);
	BBox *frame = new BBox(frameRect);
	container->AddChild(frame);
	
	BRect viewRect = frameRect;
	viewRect.OffsetTo(B_ORIGIN);
	viewRect.InsetBy(3,3);
	BRect textRect = viewRect;
	textRect.OffsetTo(B_ORIGIN);
	BTextView *textView = new RTextView(viewRect, name, textRect, 
		B_FOLLOW_NONE, B_WILL_DRAW);
	for (ulong c = 0; c < ' '; c++) {
		if (c != B_BACKSPACE && c != B_LEFT_ARROW && c != B_RIGHT_ARROW) 
			textView->DisallowChar(c);
	}
	frame->AddChild(textView);
	if (makeFocus) textView->MakeFocus();
	textView->SetFontName("Kate");
	textView->SetText(initValue);
	textView->Select(0, textView->TextLength());
}

///   MakeOptBox
//-----------------------------------------------------------------------------

BCheckBox *TFindDlg::MakeOptBox(BRect *frame, const char *label, 
	const ulong what, BView *container, bool initValue)
{
	BCheckBox *box = 
		new BCheckBox(*frame, "", label, new BMessage(what));
	container->AddChild(box);
	box->SetLowColor(light_gray_background);
	box->SetViewColor(light_gray_background);
	frame->OffsetBy(0, 17);
	if (initValue) box->SetValue(1);
	return box;
}

///   MakeDirRadio
//-----------------------------------------------------------------------------

void TFindDlg::MakeDirRadio(BRect *frame, const char *label, 
	BView *container, bool forward)
{
	BRadioButton *radio = 
		new BRadioButton(*frame, "", label, new BMessage('forw'));
	container->AddChild(radio);
	radio->SetLowColor(light_gray_background);
	radio->SetViewColor(light_gray_background);
	frame->OffsetBy(0, 17);
	if (find_info.forward == forward) radio->SetValue(1);
}

///   MakeScopeRadio
//-----------------------------------------------------------------------------

void TFindDlg::MakeScopeRadio(BRect *frame, const char *label, const ulong what, 
	BView *container, TFindScope buttonScope)
{
	BRadioButton *radio = 
		new BRadioButton(*frame, "", label, new BMessage(what));
	container->AddChild(radio);
	radio->SetLowColor(light_gray_background);
	radio->SetViewColor(light_gray_background);
	frame->OffsetBy(0, 17);
	if (scope == buttonScope) radio->SetValue(1);
	if (!haveProjectDir && buttonScope != FIND_THIS_FILE &&
		buttonScope != FIND_BE_HEADER_FILES && 
		buttonScope != FIND_STD_C_HEADER_FILES)
	{
		if (parentRef.database < 0) radio->SetEnabled(FALSE);
	}
}

///MessageReceived
//-----------------------------------------------------------------------------

void TFindDlg::MessageReceived(BMessage *msg)
{
	switch (msg->what) {	
		case 'okok':
			{
				BTextView *search = (BTextView*)FindView("search");
				BTextView *replace = (BTextView*)FindView("replace");			
				find_info.Lock();
				delete [] find_info.searchString;
				find_info.searchString = new char[search->TextLength()+1];
				strcpy(find_info.searchString, search->Text());
				delete [] find_info.replaceString;
				find_info.replaceString = new char[replace->TextLength()+1];
				strcpy(find_info.replaceString, replace->Text());
				find_info.caseSensitive = caseSensitive;
				find_info.wholeWord = wholeWord;
				find_info.wrap = wrap;
				find_info.forward = forward;
				find_info.scope = scope;
				if (scope != FIND_THIS_FILE) find_info.BuildFileList(parentRef);
				find_info.Unlock();
				parent->PostMessage(new BMessage(FIND_AGAIN),
					parent->PreferredReceiver());			
				Quit();
			}	
		case 'cncl':
			Quit();
		case 'sens':
			caseSensitive = !caseSensitive;
			break;
		case 'word':
			wholeWord = !wholeWord;
			break;
		case 'wrap':
			wrap = !wrap;
			break;
		case 'forw':
			forward = !forward;
			break;
		case 'cur ':
			scope = FIND_THIS_FILE;
			wrapCheckBox->SetEnabled(TRUE);
			break;
		case 'cpp ':
			scope = FIND_CPP_FILES;
			wrapCheckBox->SetEnabled(FALSE);
			break;
		case 'h   ':
			scope = FIND_H_FILES;
			wrapCheckBox->SetEnabled(FALSE);
			break;
		case 'cpph':
			scope = FIND_CPP_AND_H_FILES;
			wrapCheckBox->SetEnabled(FALSE);
			break;
		case 'text':
			scope = FIND_TEXT_FILES;
			wrapCheckBox->SetEnabled(FALSE);
			break;
		case 'be  ':
			scope = FIND_BE_HEADER_FILES;
			wrapCheckBox->SetEnabled(FALSE);
			break;
		case 'stdc':
			scope = FIND_STD_C_HEADER_FILES;
			wrapCheckBox->SetEnabled(FALSE);
			break;
		default:
			inherited::MessageReceived(msg);
			break;			
	}
}

///FilterKeyDown
//-----------------------------------------------------------------------------

bool TFindDlg::FilterKeyDown(ulong *aChar, BView **target)
{
	if (*aChar == B_ESCAPE) {
		// Flash Cancel button and quit if Escape key pressed.
		BButton *cancelButton = (BButton*)FindView("cancel");
		cancelButton->SetValue(1);
		snooze(100000.0);	// 0.1 seconds
		Quit();
		return FALSE;	// Superfluous - Quit doesn't return
	} else if (*aChar == B_TAB) {
		BTextView *search = (BTextView*)FindView("search");
		BTextView *replace = (BTextView*)FindView("replace");
		if (search == *target) {
			replace->Select(0, replace->TextLength());
			replace->MakeFocus();
		} else {
			search->Select(0, search->TextLength());
			search->MakeFocus();
		}
		return FALSE;
	} else {
		return TRUE;
	}
}
