//-----------------------------------------------------------------------------
//
// TGoToLineDialog.cpp
//
// "Go to Line" dialog class.
//
//-----------------------------------------------------------------------------

#include "TGoToLineDlg.h"
#include "RGlobals.h"
#include "RTextView.h"
#include "RUtil.h"
#include "TGlobals.h"

B_DEFINE_CLASS_INFO(TGoToLineDlg, BWindow);

///TGoToLineDlg
//-----------------------------------------------------------------------------

TGoToLineDlg::TGoToLineDlg(BWindow *parent, BReceiver *receiver)
	: BWindow(BRect(100, 100, 350, 200), "", B_MODAL_WINDOW, B_NOT_RESIZABLE),
	parent(parent), receiver(receiver)
{
	Lock();
	BRect bounds = Bounds();
	
	// Center dialog over parent window.
	
	BRect a = Frame();
	CenterRectOverScreenRect(a, parent->Frame());
	MoveTo(a.left, a.top);
	
	// Make the background light gray.
	
	BView *top = new BView(bounds, "", B_FOLLOW_NONE, B_WILL_DRAW);
	AddChild(top);
	top->SetViewColor(light_gray_background);
	
	// OK button.
		
	BRect okButtonRect(bounds.right-80, bounds.bottom-35,
		bounds.right-10, bounds.bottom-10);
	BButton *okButton = new BButton(okButtonRect, "",
		"OK", new BMessage('okok'));
	top->AddChild(okButton);	
	SetDefaultButton(okButton);
	
	// Cancel button.
	
	BRect cancelButtonRect = okButtonRect;
	cancelButtonRect.OffsetBy(-80, 0);
	BButton *cancelButton = new BButton(cancelButtonRect, "cancel",
		"Cancel", new BMessage('cncl'));
	top->AddChild(cancelButton);
	
	// "Go to line" label.
	
	BRect labelRect(10, okButtonRect.top-40, 80, okButtonRect.top-24);
	BStringView *label = new BStringView(labelRect, "", 
		"Go to line");
	label->SetAlignment(B_ALIGN_RIGHT);
	top->AddChild(label);
	label->SetLowColor(light_gray_background);
	label->SetFontName("Emily");
	
	// Frame around text entry field.
	
	BRect textFrameRect(labelRect.right+6, labelRect.top, 
		bounds.right-20, labelRect.bottom);
	BBox *textFrame = new BBox(textFrameRect);
	top->AddChild(textFrame);
	
	// Text entry field.
	
	BRect textViewRect = textFrameRect;
	textViewRect.OffsetTo(B_ORIGIN);
	textViewRect.InsetBy(3,3);
	BRect textRect = textViewRect;
	textRect.OffsetTo(B_ORIGIN);
	BTextView *text = new RTextView(textViewRect, "line", textRect, 
		B_FOLLOW_NONE, B_WILL_DRAW);
	for (ulong c = 0; c < 256; c++) {
		if (!isdigit(c) && c != B_BACKSPACE &&
			c != B_LEFT_ARROW && c != B_RIGHT_ARROW) 
				text->DisallowChar(c);
	}
	textFrame->AddChild(text);
	text->MakeFocus();
	text->SetFontName("Kate");
	
	Show();	
	
	Unlock();
}

///MessageReceived
//-----------------------------------------------------------------------------

void TGoToLineDlg::MessageReceived(BMessage *msg)
{
	if (msg->what == 'okok') {
		BTextView *text = (BTextView*)FindView("line");
		BMessage *msg = new BMessage(GO_TO_LINE_DOIT);
		msg->AddLong("line", atol(text->Text()));
		parent->PostMessage(msg, receiver);
		Quit();
	} else if (msg->what == 'cncl') {
		Quit();
	} else {
		inherited::MessageReceived(msg);
	}
}

///FilterKeyDown
//-----------------------------------------------------------------------------

bool TGoToLineDlg::FilterKeyDown(ulong *aChar, BView **target)
{
	if (*aChar == B_ESCAPE) {
		// Flash Cancel button and quit if Escape key pressed.
		BButton *cancelButton = (BButton*)FindView("cancel");
		cancelButton->SetValue(1);
		snooze(100000.0);	// 0.1 seconds
		Quit();
		return FALSE;	// Superfluous - Quit doesn't return
	} else {
		return TRUE;
	}
}
