//-----------------------------------------------------------------------------
//
// TUndo.cpp
//
// Undo class.
//
//-----------------------------------------------------------------------------

#include "TUndo.h"
#include "TFindInfo.h"
#include "TTextView.h"
#include "TGlobals.h"

B_DEFINE_CLASS_INFO(TUndo, BObject);

///TUndo
//-----------------------------------------------------------------------------

TUndo::TUndo(TTextView *textView)
	: textView(textView), canUndo(FALSE), inKeySeq(FALSE),
	start(0), finish(0), textToRestore(NULL)
{
}

///~TUndo
//-----------------------------------------------------------------------------

TUndo::~TUndo()
{
	delete [] textToRestore;
}

///Clear
//-----------------------------------------------------------------------------

void TUndo::Clear()
{
	canUndo = FALSE;
	inKeySeq = FALSE;
	start = finish = 0;
	delete [] textToRestore;
	textToRestore = NULL;
}

///RecordKey
//-----------------------------------------------------------------------------

void TUndo::RecordKey(ulong aChar)
{
	long newStart, newFinish;
	textView->GetSelection(&newStart, &newFinish);
	if (!inKeySeq || newStart != newFinish || newStart != finish) {
		RecordDelete(100);
		inKeySeq = TRUE;
		if (aChar == B_BACKSPACE) {
			if (newStart < newFinish) return;
			start--;
			finish--;
			*(--restoreStart) = textView->CharAt(start);
		} else {
			finish++;
		}
	} else {
		if (aChar == B_BACKSPACE) {
			finish--;
			if (finish < start) {
				start--;
				if (restoreStart <= textToRestore) {
					long len = strlen(textToRestore) + 1;
					char *newBuf = new char[len+100];
					memcpy(textToRestore+100, textToRestore, len);
					restoreStart += 100;
				}
				*(--restoreStart) = textView->CharAt(start);
			}
		} else {
			finish++;
		}
	}
}

///RecordCut
//-----------------------------------------------------------------------------

void TUndo::RecordCut()
{
	RecordDelete();
}

///RecordPaste
//-----------------------------------------------------------------------------

void TUndo::RecordPaste(BClipboard *clipboard)
{
	RecordDelete();
	clipboard->Lock();
	long numBytes;
	const char *text = clipboard->FindText(&numBytes);
	clipboard->Unlock();
	finish = start + numBytes;
}

///RecordReplace
//-----------------------------------------------------------------------------

void TUndo::RecordReplace()
{
	// Caller must have find_info lock.

	RecordDelete();
	finish = start + strlen(find_info.replaceString);
}

///RecordDelete
//-----------------------------------------------------------------------------

void TUndo::RecordDelete(long extraSpace)
{
	Clear();
	textView->GetSelection(&start, &finish);
	textToRestore = new char[finish-start+1+extraSpace];
	const char *text = textView->Text();
	memcpy(textToRestore+extraSpace, text+start, finish-start);
	textToRestore[extraSpace+finish-start] = 0;
	restoreStart = textToRestore+extraSpace;
	origStart = start;
	origFinish = finish;
	finish = start;
	canUndo = TRUE;
}

///Undo
//-----------------------------------------------------------------------------

void TUndo::Undo()
{
	textView->Select(start, finish);
	textView->Delete();
	if (textToRestore) {
		textView->Insert(restoreStart);
		textView->Select(origStart, origFinish);
	}
	textView->ScrollSelectionToMiddle();
	Clear();
}
